#!/usr/bin/env python
# coding: utf-8
from __future__ import print_function

import sys
import colour_filter
import colour_text


'''
               0   to restore default color
               1   for brighter colors
               4   for underlined text
               5   for flashing text
              30   for black foreground
              31   for red foreground
              32   for green foreground
              33   for yellow (or brown) foreground
              34   for blue foreground
              35   for purple foreground
              36   for cyan foreground
              37   for white (or gray) foreground
              40   for black background
              41   for red background
              42   for green background
              43   for yellow (or brown) background
              44   for blue background
              45   for purple background
              46   for cyan background
              47   for white (or gray) background
'''

def main():
    args = iter( sys.argv )
    prog_name = next( args )

    ct = colour_text.ColourText()
    cf = colour_filter.ColourFilter()

    ct.initTerminal()

    try:
        while True:
            arg = next( args, None )
            if arg is None:
                break

            if arg.startswith( '-' ):
                if '-debug' == arg:
                    cf.enableDebug()

                else:
                    print( ct('~error Error:~ Unknown options "%s"') % (arg,) )

            else:
                pattern = arg
                colour = next( args )
                cf.define( pattern, colour )

        cf.filterLines( sys.stdin, sys.stdout, line_buffered=True )
        return 0

    except StopIteration:
        print( ct('~error Error:~ Need more args' ) )
        return 1

    except colour_filter.ColourFilterError as e:
        print( ct('~error Error:~ %s') % (e,) )
        return 1

    except KeyboardInterrupt:
        return 0

if __name__ == '__main__':
    sys.exit( main() )



"""

setup.py for CLI-tools colour filter

"""

import setuptools
import os.path

# Use the VERSION defined in __init__.py
import colour_filter

url = 'https://github.com/barry-scott/CLI-tools'

here = os.path.abspath( os.path.dirname(__file__) )

# Get the long description from the README file
with open( os.path.join(here, 'colour_filter', 'README.md'), encoding='utf-8' ) as f:
    long_description = f.read()

def getDevStatusFromVersion():
    if 'a' in colour_filter.VERSION:
        return 'Development Status :: 3 - Alpha'

    elif 'b' in colour_filter.VERSION:
        return 'Development Status :: 4 - Beta'

    else:
        return 'Development Status :: 5 - Production/Stable'

setuptools.setup(
    name='colour-filter',

    libraries = [],

    version=colour_filter.VERSION,

    description='Colour Filter',
    long_description=long_description,
    long_description_content_type='text/markdown',

    # The project's main homepage.
    url=url,

    # Author details
    author='Barry Scott',
    author_email='barry@barrys-emacs.org',

    # Choose your license
    license='Apache 2.0',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        getDevStatusFromVersion(),

        # Indicate who your project is intended for
        'Intended Audience :: End Users/Desktop',
        'Topic :: Utilities',

        'Environment :: Console',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: Apache Software License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],

    # What does your project relate to?
    keywords='development',

    packages=['colour_filter'],
    install_requires="colour_text",
    entry_points = {
        'console_scripts': ['colour-filter=colour_filter.__main__:main'],
        }
    )



from __future__ import print_function

import sys
import os
import re

VERSION = '1.0.1'

colour_names = {
    'bold':         '1',

    'black':        '30',
    'brown':        '31',
    'green':        '32',
    'yellow':       '33',
    'blue':         '34',
    'magenta':      '35',
    'cyan':         '36',
    'gray':         '37',

    'red':          '31;1',
    'lightred':     '31;1',
    'lightgreen':   '32;1',
    'lightyellow':  '33;1',
    'lightblue':    '34;1',
    'lightmagenta,':'35;1',
    'lightcyan':    '36;1',
    'white':        '37;1',


    'bg-black':     '40',
    'bg-brown':     '41',
    'bg-green':     '42',
    'bg-yellow':    '43',
    'bg-blue':      '44',
    'bg-magenta':   '45',
    'bg-cyan':      '46',
    'bg-gray':      '47',
    'bg-white':     '47',
    '0':            '0',
    '1':            '1',
    '30':           '30',
    '31':           '31',
    '32':           '32',
    '33':           '33',
    '34':           '34',
    '35':           '35',
    '36':           '36',
    '37':           '37',
    '40':           '40',
    '41':           '41',
    '42':           '42',
    '43':           '43',
    '44':           '44',
    '45':           '45',
    '46':           '46',
    '47':           '47',
}

class ColourFilterError(Exception):
    pass

class ColourFilter:
    '''
    ColourFilter - colour parts of input lines matching regex patterns
    '''
    def __init__( self ):
        self.opt_debug = False

        self.all_patterns = []

    def enableDebug( self, enable=True ):
        self.opt_debug = enable

    def define( self, pattern, colour ):
        pattern = re.compile( pattern )

        # TBD validate colour
        # allow : instead of ';' for command ease
        all_colour_parts = colour.replace( ':', ';' ).split(';')

        # replace all the colour names with with the escape codes
        all_colour_parts = [colour_names.get(colour, '0') for colour in all_colour_parts]

        self.all_patterns.append( (pattern, ';'.join(all_colour_parts)) )

    def filterLines( self, input_file, output_file, line_buffered ):
        while True:
            line = input_file.readline()
            if line == '':
                break

            # list of tuples of (coloured, text)
            # only colour in text that is not already coloured
            line_parts = [(False, line)]

            for pattern, colour in self.all_patterns:
                index = 0

                while index < len(line_parts):
                    coloured, text = line_parts[index]

                    if coloured:
                        index += 1
                        continue

                    match = pattern.search( text )
                    # no match or match is zero width for patterns like [0-9]*
                    if match is None or match.start(0) == match.end(0):
                        index += 1
                        continue

                    pre = text[0:match.start(0)]
                    middle = text[match.start(0):match.end(0)]
                    post = text[match.end(0):]

                    if len(pre) > 0:
                        line_parts.insert( index, (False, pre) )
                        index += 1

                    line_parts[index] = (True, '\033[%sm%s\033[m' % (colour, middle))
                    index += 1

                    if len(post) > 0:
                        line_parts.insert( index, (False, post) )

            output_file.write( ''.join( [text for coloured, text in line_parts] ) )
            if line_buffered:
                output_file.flush()

    def debug( self, msg ):
        if self.opt_debug:
            print( 'Debug: %s' % (msg,) )
            sys.stdout.flush()
